//+------------------------------------------------------------------+
//|                                                        Trade.mqh |
//|                      Copyright  2009, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Class CTrade.                                                    |
//| Appointment: Class trade operations.                             |
//+------------------------------------------------------------------+
class CTrade
  {
protected:
   MqlTradeRequest   m_request;         // request data
   MqlTradeResult    m_result;          // result data
   ulong             m_magic;           // expert magic number
   ulong             m_deviation;       // deviation default
public:
                     CTrade();
   //--- methods of access to protected data
   uint              ResultRetcode() { return(m_result.retcode); }
   string            ResultRetcodeDescription();
   ulong             ResultDeal()    { return(m_result.deal);    }
   ulong             ResultOrder()   { return(m_result.order);   }
   double            ResultVolume()  { return(m_result.volume);  }
   double            ResultPrice()   { return(m_result.price);   }
   double            ResultBid()     { return(m_result.bid);     }
   double            ResultAsk()     { return(m_result.ask);     }
   string            ResultComment() { return(m_result.comment); }
   //--- trade methods
   void              SetExpertMagicNumber(ulong magic)     { m_magic=magic;         }
   void              SetDeviationInPoints(ulong deviation) { m_deviation=deviation; }
   //--- methods for working with positions
   bool              PositionOpen(const string symbol,ENUM_ORDER_TYPE order_type,double volume,
                                  double price,double sl,double tp,const string comment="");
   bool              PositionModify(const string symbol,double sl,double tp);
   bool              PositionClose(const string symbol,ulong deviation=ULONG_MAX);
   //--- methods for working with pending orders
   bool              OrderOpen(const string symbol,ENUM_ORDER_TYPE order_type,double volume,
                               double limit_price,double price,double sl,double tp,
                               ENUM_ORDER_TYPE_TIME type_time,datetime expiration,
                               const string comment="");
   bool              OrderModify(ulong ticket,double price,double sl,double tp,
                                 ENUM_ORDER_TYPE_TIME type_time,datetime expiration);
   bool              OrderDelete(ulong ticket);
   //--- methods for display request/result information
   void              PrintRequest();
   void              PrintResult();
  };
//+------------------------------------------------------------------+
//| Constructor CTrade.                                              |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CTrade::CTrade()
  {
//--- initialize protected data
   ZeroMemory(m_request);
   ZeroMemory(m_result);
   m_magic=0;
   m_deviation=10;
  }
//+------------------------------------------------------------------+
//| Get the retcode value as string.                                 |
//| INPUT:  no.                                                      |
//| OUTPUT: the retcode value as string.                             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CTrade::ResultRetcodeDescription()
  {
   string str;
//---
   switch(m_result.retcode)
     {
      case TRADE_RETCODE_REQUOTE:
         str="Requote";
         break;
      case TRADE_RETCODE_REJECT:
         str="Request rejected";
         break;
      case TRADE_RETCODE_CANCEL:
         str="Request cancelled by trader";
         break;
      case TRADE_RETCODE_PLACED:
         str="Order placed";
         break;
      case TRADE_RETCODE_DONE:
         str="Request done";
         break;
      case TRADE_RETCODE_DONE_PARTIAL:
         str="Request done partially";
         break;
      case TRADE_RETCODE_ERROR:
         str="Common error";
         break;
      case TRADE_RETCODE_TIMEOUT:
         str="Request cancelled by timeout";
         break;
      case TRADE_RETCODE_INVALID:
         str="Invalid request";
         break;
      case TRADE_RETCODE_INVALID_VOLUME:
         str="Invalid volume in request";
         break;
      case TRADE_RETCODE_INVALID_PRICE:
         str="Invalid price in request";
         break;
      case TRADE_RETCODE_INVALID_STOPS:
         str="Invalid stop(s) request";
         break;
      case TRADE_RETCODE_TRADE_DISABLED:
         str="Trade is disabled";
         break;
      case TRADE_RETCODE_MARKET_CLOSED:
         str="Market is closed";
         break;
      case TRADE_RETCODE_NO_MONEY:
         str="Not enough money";
         break;
      case TRADE_RETCODE_PRICE_CHANGED:
         str="Price changed";
         break;
      case TRADE_RETCODE_PRICE_OFF:
         str="No quotes for query processing";
         break;
      case TRADE_RETCODE_INVALID_EXPIRATION:
         str="Invalid expiration time in request";
         break;
      case TRADE_RETCODE_ORDER_CHANGED:
         str="Order state changed";
         break;
      case TRADE_RETCODE_TOO_MANY_REQUESTS:
         str="Too frequent requests";
         break;
      default:
         str="Unknown result";
     }
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| Open position.                                                   |
//| INPUT:  symbol     -symbol for trade,                            |
//|         order_type -direct for open,                             |
//|         volume     -volume of position,                          |
//|         price      -price for open,                              |
//|         stop       -price of stop loss,                          |
//|         take       -price of take profit,                        |
//|         comment    -comment of position.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTrade::PositionOpen(const string symbol,ENUM_ORDER_TYPE order_type,double volume,
                          double price,double sl,double tp,const string comment)
  {
//--- checking
   if(order_type!=ORDER_TYPE_BUY && order_type!=ORDER_TYPE_SELL)
     {
      m_result.retcode=TRADE_RETCODE_INVALID;
      m_result.comment="Invalid order type";
      return(false);
     }
//--- setting request
   m_request.action      =TRADE_ACTION_DEAL;
   m_request.symbol      =symbol;
   m_request.magic       =m_magic;
   m_request.volume      =volume;
   m_request.type        =order_type;
   m_request.price       =price;
   m_request.sl          =sl;
   m_request.tp          =tp;
   m_request.deviation   =m_deviation;
   m_request.type_filling=ORDER_FILLING_AON;
   m_request.comment     =comment;
//---
   return(OrderSend(m_request,m_result));
  }
//+------------------------------------------------------------------+
//| Modify specified opened position.                                |
//| INPUT:  symbol -symbol for trade,                                |
//|         stop   -new price of stop loss,                          |
//|         take   -new price of take profit,                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTrade::PositionModify(const string symbol,double sl,double tp)
  {
//--- setting request
   m_request.action=TRADE_ACTION_SLTP;
   m_request.symbol=symbol;
   m_request.sl    =sl;
   m_request.tp    =tp;
//---
   return(OrderSend(m_request,m_result));
  }
//+------------------------------------------------------------------+
//| Close specified opened position.                                 |
//| INPUT:  symbol    -symbol for trade,                             |
//|         deviation -deviation for price close.                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTrade::PositionClose(const string symbol,ulong deviation)
  {
   double price;
//--- checking
   if(PositionSelect(symbol))
     {
      if(PositionGetInteger(POSITION_TYPE)==ORDER_TYPE_BUY)
        {
         //--- prepare query for close BUY position
         m_request.type =ORDER_TYPE_SELL;
         m_request.price=SymbolInfoDouble(symbol,SYMBOL_BID);
        }
      else
        {
         //--- prepare query for close SELL position
         m_request.type =ORDER_TYPE_BUY;
         m_request.price=SymbolInfoDouble(symbol,SYMBOL_ASK);
        }
     }
//--- setting request
   m_request.action      =TRADE_ACTION_DEAL;
   m_request.symbol      =symbol;
   m_request.volume      =PositionGetDouble(POSITION_VOLUME);
   m_request.sl          =0.0;
   m_request.tp          =0.0;
   m_request.deviation   =(deviation==ULONG_MAX) ? m_deviation : deviation;
   m_request.type_filling=ORDER_FILLING_AON;
//---
   return(OrderSend(m_request,m_result));
  }
//+------------------------------------------------------------------+
//| Installation pending order.                                      |
//| INPUT:  symbol     -symbol for trade,                            |
//|         order_type -type of order,                               |
//|         volume     -volume of order,                             |
//|         limit_price-limit price for activate order,              |
//|         price      -price for open,                              |
//|         sl         -price of stop loss,                          |
//|         tp         -price of take profit,                        |
//|         type_time  -type expiration,                             |
//|         expiration -time expiration,                             |
//|         comment    -comment of order.                            |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTrade::OrderOpen(const string symbol,ENUM_ORDER_TYPE order_type,double volume,double limit_price,
                       double price,double sl,double tp,
                       ENUM_ORDER_TYPE_TIME type_time,datetime expiration,const string comment)
  {
//--- checking
   if(order_type==ORDER_TYPE_BUY || order_type==ORDER_TYPE_SELL)
     {
      m_result.retcode=TRADE_RETCODE_INVALID;
      m_result.comment="Invalid order type";
      return(false);
     }
//--- setting request
   m_request.action      =TRADE_ACTION_PENDING;
   m_request.symbol      =symbol;
   m_request.magic       =m_magic;
   m_request.volume      =volume;
   m_request.type        =order_type;
   m_request.price       =price;
   m_request.sl          =sl;
   m_request.tp          =tp;
   m_request.type_filling=ORDER_FILLING_AON;
   m_request.type_time   =type_time;
   m_request.expiration  =expiration;
   m_request.comment     =comment;
//---
   return(OrderSend(m_request,m_result));
  }
//+------------------------------------------------------------------+
//| Modify specified pending order.                                  |
//| INPUT:  ticket     -ticket for modify,                           |
//|         price      -new price for open,                          |
//|         sl         -new price of stop loss,                      |
//|         tp         -new price of take profit,                    |
//|         type_time  -new type expiration,                         |
//|         expiration -new time expiration.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTrade::OrderModify(ulong ticket,double price,double sl,double tp,ENUM_ORDER_TYPE_TIME type_time,datetime expiration)
  {
//--- setting request
   m_request.action      =TRADE_ACTION_MODIFY;
   m_request.order       =ticket;
   m_request.price       =price;
   m_request.sl          =sl;
   m_request.tp          =tp;
   m_request.type_time   =type_time;
   m_request.expiration  =expiration;
//---
   return(OrderSend(m_request,m_result));
  }
//+------------------------------------------------------------------+
//| Delete specified pending order.                                  |
//| INPUT:  ticket -ticket of order for delete.                      |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTrade::OrderDelete(ulong ticket)
  {
//--- setting request
   m_request.action    =TRADE_ACTION_REMOVE;
   m_request.order     =ticket;
   m_request.symbol    =NULL;
   m_request.magic     =0;
   m_request.volume    =0.0;
   m_request.type      =0;
   m_request.price     =0.0;
   m_request.sl        =0.0;
   m_request.tp        =0.0;
   m_request.type_time =0;
   m_request.expiration=0;
//---
   return(OrderSend(m_request,m_result));
  }
//+------------------------------------------------------------------+
//| Output full information of request to log.                       |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CTrade::PrintRequest()
  {
   printf("Request");
   printf("%d,%I64u,%I64u,'%s',%f,%f,%f,%f,%f,%I64u,%d,%d,%d,'%s','%s'",
           m_request.action,m_request.magic,m_request.order,
           m_request.symbol,m_request.volume,m_request.price,
           m_request.stoplimit,m_request.sl,m_request.tp,
           m_request.deviation,m_request.type,m_request.type_filling,
           m_request.type_time,TimeToString(m_request.expiration),m_request.comment);
  }
//+------------------------------------------------------------------+
//| Output full information of result to log.                        |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CTrade::PrintResult(void)
  {
   printf("Result");
   printf("%d,%I64u,%I64u,%f,%f,%f,%f,'%s'",
           m_result.retcode,m_result.deal,m_result.order,
           m_result.volume,m_result.price,m_result.bid,
           m_result.ask,m_result.comment);
  }
//+------------------------------------------------------------------+
